#!perl -w
use Test::More tests => 14;

use strict;
use DE_EPAGES::Core::API::File qw (
    CopyFile
    DeleteDirectory
    ExistsDirectory
    CopyDirectory
    GetFileNames
    GetFileMD5Digest
);
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Object::API::XMLCompare qw ( CompareXML );
use DE_EPAGES::Object::API::XMLStructureImportHandler;
use DE_EPAGES::Installer::API::LangDiffBuilder qw (
    GetTargetEncoding
    PostProcessXMLFile
    AddFilePrefix
    ChangeStructLanguage
    GetMainLanguage
    ReplaceLanguageAttribute
    ReplaceLanguageFileTag
    IsLanguageSupported
);

sub TestBuildTranslationPackage {
    # EPB-168
    my $Diff_de = './diff_de';
    my $Diff_en = './diff_en';
    my $Temp_en = './tmp_diff_en';

    DeleteDirectory($Temp_en, { recursive => 1 }) if(ExistsDirectory($Temp_en));
    my $Cmd = $ENV{'PERL'} . " ../Scripts/buildTranslationPackage.pl -source $Diff_de -target $Temp_en -lang en -nozip -nooutput";
    system($Cmd) == 0 or die 'buildTranslationPackage.pl failed';

    my @ExpectedFiles = (
        "$Temp_en/DE_EPAGES/Billsafe/Templates/transimp_Dictionary.en.xml",
        "$Temp_en/DE_EPAGES/Billsafe/Templates/SF/transimp_SF.INC-BillsafeResult.en.xml",
        "$Temp_en/DE_EPAGES/Content/Database/XML/transimp_Translation.Actions.en.xml",
    );

    my @TmpFiles = grep { $_ !~ /\/CVS\//; } @{GetFileNames($Temp_en , {'recursive' => 1})};

    my $CompareFiles = sub {
        foreach my $Path (@ExpectedFiles) {
            next if($Path =~ /\/CVS\//);
            # expected file must be present in TmpFiles
            unless(ElementOf($Path, @TmpFiles)) {
                print("$Path missing\n");
                return 0 ;
            }

            my $DiffPath = $Path;
            $DiffPath =~ s/^\Q$Temp_en\E/$Diff_en/;
            if($DiffPath =~ /transimp_(?:.+)\.xml$/) {
                # Language must have been changed to 'en'
                my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
                my $ExpectedStructure = $ImportHandler->getXMLStructure($Path, { 'register_notrans' => 1});
                my $OutputStructure = $ImportHandler->getXMLStructure($DiffPath, { 'register_notrans' => 1});

                my $ExpectedLanguage = $ExpectedStructure->getTagAttribute('Language', 'Language');
                my $OutputLanguage = $OutputStructure->getTagAttribute('Language', 'Language');

                return 0 unless($ExpectedLanguage eq $OutputLanguage && $ExpectedLanguage eq 'en');
            }
            else
            {
                # non-xml files must be the same
                return 0 unless(GetFileMD5Digest($Path) eq GetFileMD5Digest($DiffPath));
            }

        }
        return 1;
    };

    # 13
    my $Result = scalar(@ExpectedFiles) == scalar(@TmpFiles) && $CompareFiles->();
    DeleteDirectory($Temp_en, { recursive => 1 }) if(ExistsDirectory($Temp_en));
    return $Result;
}

sub TestMergeTranslationPackage {
    # EPB-168
    my $Diff_de = './diff_de';
    my $Diff_en = './diff_en';
    my $Diff_es = './diff_es';
    my $Temp_es = './tmp_diff_es';

    DeleteDirectory($Temp_es, { recursive => 1 }) if(ExistsDirectory($Temp_es));
    CopyDirectory($Diff_es, $Temp_es, undef, { recursive => 1 });
    die "Could not create $Temp_es" unless(ExistsDirectory($Temp_es));

    my $Cmd = $ENV{'PERL'} . " ../Scripts/mergeTranslationPackage.pl -source $Diff_en -target $Temp_es -diff $Diff_de -lang es -nooutput";
    system($Cmd) == 0 or die 'mergeTranslationPackage.pl failed';

    my @ExpectedFiles = (
        "$Temp_es/DE_EPAGES/Billsafe/SupportedLanguages.txt",
        "$Temp_es/DE_EPAGES/Billsafe/Templates/transimp_Dictionary.es.xml",
        "$Temp_es/DE_EPAGES/Billsafe/Templates/SF/transimp_SF.INC-BillsafeResult.es.xml",
        "$Temp_es/DE_EPAGES/Content/Database/XML/transimp_Translation.Actions.es.xml",
        "$Temp_es/DE_EPAGES/RealexPayment/Templates/MBO/transdel_MBO-PaymentMethodRealexPaymentSettings.TabPage.xx.xml",
        "$Temp_es/DE_EPAGES/RealexPayment/SupportedLanguages.txt",
    );

    my @TmpFiles = grep { $_ !~ /\/CVS\//; } @{GetFileNames($Temp_es , {'recursive' => 1})};

    my $CompareFiles = sub {
        foreach my $Path (@ExpectedFiles) {
            # expected file must be present in TmpFiles
            unless(ElementOf($Path, @TmpFiles)) {
                print("$Path missing\n");
                return 0 ;
            }

            if($Path =~ /transimp_(?:.+)\.xml$/) {
                # Language must have been changed to 'es'
                my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
                my $OutputStructure = $ImportHandler->getXMLStructure($Path, { 'register_notrans' => 1});
                my $OutputLanguage = $OutputStructure->getTagAttribute('Language', 'Language');

                unless($OutputLanguage eq 'es') {
                    print("Expected language 'es', but found '$OutputLanguage'\n");
                    return 0;
                }
            }
        }
        return 1;
    };

    my $Result = scalar(@ExpectedFiles) == scalar(@TmpFiles) && $CompareFiles->();
    DeleteDirectory($Temp_es, { recursive => 1 }) if(ExistsDirectory($Temp_es));
    return $Result;
}

sub Main {
    # 1..4
    ok(GetTargetEncoding('test.ru.xml') eq 'iso-8859-5', 'GetTargetEncoding(ru)');
    ok(GetTargetEncoding('test.xx.xml', 0) eq 'utf8', 'GetTargetEncoding(xx)');
    ok(AddFilePrefix('filename', 'prefix') eq './prefixfilename', 'AddFilePrefix(filename)');
    ok(AddFilePrefix('/this/is/a/test.ru.xml', 'prefix_') eq '/this/is/a/prefix_test.ru.xml',
        'AddFilePrefix(/path/to/filename)');

    # 5
    my $PostprocessInput = 'LangDiff_Postprocess_in.en.xml';
    my $PostprocessOutput = 'LangDiff_Postprocess_out.en.xml';
    my $PostprocessExpected = 'LangDiff_Postprocess_expected.en.xml';
    unlink($PostprocessOutput);
    CopyFile($PostprocessInput, $PostprocessOutput);
    PostProcessXMLFile($PostprocessOutput);
    ok(CompareXML($PostprocessOutput, $PostprocessExpected), 'PostProcess');
    unlink($PostprocessOutput);

    # 6
    my $ChangeLangInput = 'LangDiff_Postprocess_in.en.xml';
    my $ChangeLangOutput = 'LangDiff_Postprocess_out.de.xml';
    my $ChangeLangExpected = 'LangDiff_ChangeLanguage_expected.de.xml';
    my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $Structure = $ImportHandler->getXMLStructure($ChangeLangInput, { 'register_notrans' => 1} );
    ChangeStructLanguage($Structure, 'de');
    my $ExportDriver = DE_EPAGES::Object::API::XMLStructureExportDriver->new();
    unlink($ChangeLangOutput);
    $ExportDriver->exportXML($ChangeLangOutput, $Structure,
        { encoding => GetTargetEncoding($ChangeLangOutput), quotecharacter => '"' });
    ok(CompareXML($ChangeLangOutput, $ChangeLangExpected), 'ChangeLanguage');
    unlink($ChangeLangOutput);

    # 7
    is(GetMainLanguage('en-US'), 'en', 'GetMainLanguage en-US');
    # 8
    is(GetMainLanguage('en'), 'en', 'GetMainLanguage en');

    # 9
    is(ReplaceLanguageAttribute('<Language foo="42" Language="de" bar="23" />' . "\r\n", 'en'),
        '<Language foo="42" Language="en" bar="23" />' . "\r\n", 'ReplaceLanguageAttribute');

    # 10
    is(ReplaceLanguageFileTag('transimp_MBO-AmazonConfigGeneral.TabPage.de.xml', 'en'),
        'transimp_MBO-AmazonConfigGeneral.TabPage.en.xml', 'ReplaceLanguageFileTag');

    # 11
    ok(IsLanguageSupported($ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Amazon', 'de'), 'IsLanguageSupported');
    # 12
    ok(!IsLanguageSupported($ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Diagnostics', 'ca'), 'not IsLanguageSupported');

    # 13
    ok(TestBuildTranslationPackage(), 'Built language package has expected content');
    # 14
    ok(TestMergeTranslationPackage(), 'Merged language package has expected content');

}

RunScript( Sub => \&Main );

1;
